from typing import Sequence

from .gateway import Gateway
from .proxy import Proxy
from .sensor import (
    Measurement,
    MeasurementError,
    MeasurementType,
    Sensor,
    WindDirection,
)

__all__: Sequence[str] = [
    "Gateway",
    "Proxy",
    "Measurement",
    "MeasurementError",
    "MeasurementType",
    "Sensor",
    "WindDirection",
]



"""MobileAlerts proxy."""

from typing import Any, Dict, List

import logging
import socket
import struct
import time

from aiohttp import web

from .gateway import Gateway, SensorHandler

_LOGGER = logging.getLogger(__name__)


class Proxy:
    """Proxy server to read PUT requests of Mobile Alerts/WeatherHub sensors."""

    def __init__(
        self,
        handler: SensorHandler,
        local_ip_address: str = "",
        local_port: int = 0,
    ) -> None:
        self._handler = handler
        self._gateways: Dict[str, Gateway] = dict()
        self._local_ip_address = local_ip_address
        self._local_port = local_port

    async def _start(
        self,
    ) -> None:
        """Internal start proxy server."""
        self._site = web.TCPSite(self._runner, self._local_ip_address, self._local_port)
        await self._site.start()
        server: Any = self._site._server
        sockets: List[socket.socket] = server._sockets
        address = sockets[0].getsockname()
        self._host = str(address[0])
        self._port = int(address[1])
        _LOGGER.debug("Proxy started on address %s:%s", self._host, self._port)

    async def start(
        self,
    ) -> None:
        """Start proxy server."""
        self._server = web.Server(self.request_handler)
        self._runner = web.ServerRunner(self._server)
        await self._runner.setup()
        await self._start()

    async def stop(
        self,
    ) -> None:
        """Stop proxy server and detach all attached gateways."""
        await self._site.stop()
        self.detach_all_gateways()

    async def restart(
        self,
        local_ip_address: str = "",
        local_port: int = 0,
    ) -> None:
        """Restart proxy server on new IP address and port."""
        await self._site.stop()
        self._local_ip_address = local_ip_address
        self._local_port = local_port
        await self._start()

    async def request_handler(self, request: web.BaseRequest) -> web.StreamResponse:
        response = await self.send_response_to_gateway(request)
        if request.method == "PUT":
            headers = request.headers
            if (
                ("Content-Type" in headers)
                and (headers["Content-Type"] == "application/octet-stream")
                and ("Content-Length" in headers)
            ):
                if "HTTP_IDENTIFY" in headers:
                    identify = headers["HTTP_IDENTIFY"].split(":")
                    if len(identify) == 3:
                        gateway: Gateway = self._gateways[identify[1]]
                        content = await request.content.read(
                            int(request.headers["Content-Length"])
                        )
                        await gateway.handle_update(identify[2], content)
                        await gateway.resend_data_to_cloud(
                            request.rel_url, headers, content
                        )
                    else:
                        _LOGGER.error(
                            "Invalid HTTP_IDENTIFY header in gateway's PUT request"
                        )
                else:
                    _LOGGER.error("No HTTP_IDENTIFY header in gateway's PUT request")
            else:
                _LOGGER.error("Invalid content in gateway's PUT request")
        else:
            _LOGGER.error("Invalid gateway's request method %s", request.method)
        return response

    async def send_response_to_gateway(
        self, request: web.BaseRequest
    ) -> web.StreamResponse:
        response = web.StreamResponse(
            headers={
                "Content-Type": "application/octet-stream",
                "Content-Length": "24",
                "Connection": "close",
            }
        )
        await response.prepare(request)
        content = struct.pack(">IIIIII", 1, 0, int(time.time()), 1, 0x1761D480, 1)
        await response.write_eof(content)
        _LOGGER.debug("Response to gateway: %s", content.hex().upper())
        return response

    def attach_gateway(self, gateway: Gateway) -> None:
        """Attach the gateway to proxy server."""
        if gateway.gateway_id not in self._gateways:
            self._gateways[gateway.gateway_id] = gateway
            gateway.attach_to_proxy(self._host, self._port, self._handler)

    def detach_gateway(self, gateway: Gateway) -> None:
        """Detach the gateway from proxy server."""
        if gateway.gateway_id in self._gateways:
            gateway.detach_from_proxy()
            self._gateways.pop(gateway.gateway_id)

    def detach_all_gateways(self) -> None:
        """Detach from proxy server all attached gateways."""
        for gateway in self._gateways.values():
            gateway.detach_from_proxy()
        self._gateways.clear()

    def __del__(self) -> None:
        self.detach_all_gateways()

    @property
    def gateways(self):
        return self._gateways.values()

    @property
    def host(self) -> str:
        return self._host

    @property
    def port(self) -> int:
        return self._port



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mobilealerts']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8,<4.0']

extras_require = \
{':python_version < "3.8"': ['importlib_metadata>=4.5.0,<5.0.0']}

setup_kwargs = {
    'name': 'mobilealerts',
    'version': '0.1.2',
    'description': 'Python classes for MobileAlerts gateway and sensors',
    'long_description': '# python-mobilealerts\n\n[![build](https://github.com/PlusPlus-ua/python-mobilealerts/actions/workflows/build.yml/badge.svg)](https://github.com/PlusPlus-ua/python-mobilealerts/actions/workflows/build.yml) [![Python Version](https://img.shields.io/pypi/pyversions/mobilealerts.svg)](https://pypi.org/project/mobilealerts/) [![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/PlusPlus-ua/python-mobilealerts/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot) [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black) [![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit) [![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/PlusPlus-ua/python-mobilealerts/blob/master/.pre-commit-config.yaml) [![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/PlusPlus-ua/python-mobilealerts/releases) [![License](https://img.shields.io/github/license/PlusPlus-ua/python-mobilealerts)](https://github.com/PlusPlus-ua/python-mobilealerts/blob/master/LICENSE)\n\n\n## Python classes for MobileAlerts gateway and sensors\n\nLibrary act as proxy server which intercepts all measurements and sensors properties which sends MobileAlerts gateway to the cloud. \n\nEverything works in asynchronous way in order to build on the base of this library [Home assistant](https://github.com/home-assistant/core) integration.\n\n## Installation\n\n```bash\npip install -U python-mobilealerts\n```\n\nor install with `Poetry`\n\n```bash\npoetry add python-mobilealerts\n```\n\n## 🛡 License\n\n[![License](https://img.shields.io/github/license/PlusPlus-ua/python-mobilealerts)](https://github.com/PlusPlus-ua/python-mobilealerts/blob/master/LICENSE)\n\nThis project is licensed under the terms of the `MIT` license. See [LICENSE](https://github.com/PlusPlus-ua/python-mobilealerts/blob/master/LICENSE) for more details.\n\n## Credits\n\nThis project was inspired and developed on the base of [MMMMobileAlerts](https://github.com/sarnau/MMMMobileAlerts)\n\nThis project was generated with [python-package-template](https://github.com/TezRomacH/python-package-template)\n',
    'author': 'PlusPlus-ua',
    'author_email': 'alexplas@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/PlusPlus-ua/python-mobilealerts',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
